<?php

namespace Drupal\murmurations\Form;

use Drupal\murmurations\MurmurationsPluginManager;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\UserData;
use Drupal\Core\Render\Markup;
use Drupal\Core\Routing\CurrentRouteMatch;
use Symfony\Component\HttpFoundation\Request;

abstract class  FilterForm extends FormBase {

  /**
   * @var Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * @var UserData
   */
  protected $userData;

  /**
   * @var MurmurationsPluginManager
   */
  protected $plugin;

  /**
   * @var CurrentRouteMatch
   */
  protected $route_match;

  /**
   * @param UserData $user_data
   * @param Request $request
   * @param CurrentRouteMatch $route_match
   * @param MurmurationsPluginManager $murm_plugin_manager
   */
  public function __construct(UserData $user_data, Request $request, CurrentRouteMatch $route_match, MurmurationsPluginManager $murm_plugin_manager) {
    $this->userData = $user_data;
    $this->request = $request;
    $this->routeMatch = $route_match;
    $plugin_id = $request->attributes->get('_route_object')->getOptions()['plugin'];
    $this->plugin = $murm_plugin_manager->createInstance($plugin_id);
  }

  /**
   * {@inheritDoc}
   */
  function buildForm(array $form, FormStateInterface $form_state) {
    $params = $this->request->query->all();
    $form['basic'] = [
      '#type' => 'container',
      'range' => [
        '#title' => $this->t('Max distance'),
        '#description' => $this->t('from me (0 means any)'),
        '#type' => 'number',
        '#default_value' => $this->userData->get('murmurations', $this->currentUser()->id(), 'range')??5,
        '#field_suffix' => 'km',
        '#access' => (bool)$this->config('murmurations.settings')->get('fallback_point.lat'), // ONly show
        '#weight' => 1,
        '#size' => 6
      ],
      'fragment' => [
        '#title' => $this->t('Keywords'),
        '#description' => $this->t('Separated by commas for OR'),
        '#type' => 'textfield',
        '#default_value' => $params['fragment']??'',
        '#weight' => 2,
        '#size' => 10
      ],
      '#weight' => 1
    ];

    $uid = $this->currentUser()->id();
    $this->plugin->filterFormAlter(
      $form,
      $form_state,
      $params + (array)$this->userData->get('murmurations', $uid, 'pref')
    );

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
      '#weight' => 10
    ];

    if ($params) {
      if ($response = $this->searchRequest($params)) {
        $form['results'] = [
          '#title' => $this->t('@num results', ['@num' => $response[1]->number_of_results]),
          '#type' => 'fieldset',
          '#weight' => 12
        ];
        $form['results']['items'] = $response[0] ?
          $this->showResults($response[0], $response[1]->number_of_results) :
          ['#markup' => Markup::create($this->t('No results.'))];
      }
    }

    $form['#attached']['library'][] = 'murmurations/filter_form';
    return $form;
  }

  /**
   * {@inheritDoc}
   */
  function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
    $filters = $this->plugin->filterFormValues($form_state->cleanValues()->getvalues());
    if ($range = $form_state->getValue('range')) {
      $filters['range'] = $range;
      // All searches should be centred around a point.
      $filters += $this->getCoords();
    }
    if ($fragment = $form_state->getValue('fragment')) {
      $filters['text'] = $fragment;
    }
    $form_state->setRedirect(
      $this->routeMatch->getRouteName(),
      [],
      [
        'query' => array_filter($filters)
      ]
    );
    // Remember the user settings.
    $this->userData
     ->set('murmurations', $this->currentUser()->id(), 'range', $range);
  }
  /**
   * Get the centre of the search, the user's position or the site position.
   * @return array
   *   with float 'lat' and float 'lon'
   */
  function getCoords() : array {
    return \Drupal::config('murmurations.settings')->get('fallback_point');
  }


  /**
   * Submit to the search engine and parse the results a bit
   *
   * @param array $params
   *   with any of keys string type (required), string text, float radius, bool directexchange, bool indirectexchange, bool money, lat/lon from
   *
   * @return array
   *   the results and the metadata.
   */
  public function searchRequest(array $params) : array {
    $schema = $this->plugin->getPluginDefinition()['schema'];
    $options['headers'] = [
      'Accept' => 'application/json',
    ];
    $aggregator_url = $this->plugin->getConfiguration()['aggregator_url'];
    $aggregator_url .= $schema.'?'. http_build_query($params);
    try {
      $response = \Drupal::service('http_client')->get($aggregator_url, $options);
    }
    catch(\Exception $e)  {
      $message = "No response from $aggregator_url";
      trigger_error($message, E_USER_ERROR);
      $response = new \GuzzleHttp\Psr7\Response(404, [], NULL, NULL, $message);
    }

    if ($response->getStatusCode() != 200) {
      \Drupal::messenger()->addStatus($response->getReasonPhrase());
      return [];
    }
    $contents = json_decode($response->getBody()->getContents());
    return [(array)$contents->data, $contents->meta];
  }

  abstract public function showResults(array $results, int $number_of_results) :array;

}

